"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var server_1 = require("./server");
var chai = require("chai");
var chaiAsPromised = require("chai-as-promised");
var fs = require("fs");
var util = require("util");
var path = require("path");
var q = require("q");
var _ = require("underscore");
var mkdirp = require("mkdirp");
var rimraf = require("rimraf");
chai.use(chaiAsPromised);
var expect = chai.expect;
var rm = util.promisify(rimraf);
describe('Server Spec', function () {
    var server;
    var monkeypatchFile = path.resolve(__dirname, '../test/monkeypatch.rb');
    var absolutePath;
    var relativePath;
    beforeEach(function () {
        relativePath = ".tmp/" + Math.floor(Math.random() * 1000);
        absolutePath = path.resolve(__dirname, '..', relativePath);
        mkdirp.sync(absolutePath);
    });
    afterEach(function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!server) return [3, 2];
                    return [4, server.delete()];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    if (!fs.existsSync(absolutePath)) return [3, 4];
                    return [4, rm(absolutePath)];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4: return [2];
            }
        });
    }); });
    var relativeSSLCertPath = 'test/ssl/server.crt';
    var absoluteSSLCertPath = path.resolve(__dirname, '..', relativeSSLCertPath);
    var relativeSSLKeyPath = 'test/ssl/server.key';
    var absoluteSSLKeyPath = path.resolve(__dirname, '..', relativeSSLKeyPath);
    describe('Start server', function () {
        context('when no options are set', function () {
            it('should start correctly with defaults', function () {
                server = server_1.default();
                return expect(server.start()).to.eventually.be.fulfilled;
            });
        });
        context('when invalid options are set', function () {
            it('should fail if custom ssl certs do not exist', function () {
                expect(function () {
                    return server_1.default({
                        ssl: true,
                        sslcert: 'does/not/exist',
                        sslkey: absoluteSSLKeyPath,
                    });
                }).to.throw(Error);
            });
            it('should fail if custom ssl keys do not exist', function () {
                expect(function () {
                    return server_1.default({
                        ssl: true,
                        sslcert: absoluteSSLCertPath,
                        sslkey: 'does/not/exist',
                    });
                }).to.throw(Error);
            });
            it("should fail if custom ssl cert is set, but ssl key isn't", function () {
                expect(function () {
                    return server_1.default({
                        ssl: true,
                        sslcert: absoluteSSLCertPath,
                    });
                }).to.throw(Error);
            });
            it("should fail if custom ssl key is set, but ssl cert isn't", function () {
                expect(function () {
                    return server_1.default({
                        ssl: true,
                        sslkey: absoluteSSLKeyPath,
                    });
                }).to.throw(Error);
            });
        });
        context('when valid options are set', function () {
            it('should start correctly when instance is delayed', function () {
                server = server_1.default();
                var waitForServerUp = server['__waitForServiceUp'].bind(server);
                return q
                    .allSettled([
                    waitForServerUp(server.options),
                    q.delay(5000).then(function () { return server.start(); }),
                ])
                    .then(function (results) {
                    return expect(_.reduce(results, function (m, r) { return m && r.state === 'fulfilled'; }))
                        .to.be.true;
                });
            });
            it('should start correctly with ssl', function () {
                server = server_1.default({ ssl: true });
                expect(server.options.ssl).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with custom ssl cert/key', function () {
                server = server_1.default({
                    ssl: true,
                    sslcert: absoluteSSLCertPath,
                    sslkey: absoluteSSLKeyPath,
                });
                expect(server.options.ssl).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with custom ssl cert/key but without specifying ssl flag', function () {
                server = server_1.default({
                    sslcert: absoluteSSLCertPath,
                    sslkey: absoluteSSLKeyPath,
                });
                expect(server.options.ssl).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with cors', function () {
                server = server_1.default({ cors: true });
                expect(server.options.cors).to.equal(true);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with port', function () {
                var port = Math.floor(Math.random() * 999) + 9000;
                server = server_1.default({ port: port });
                expect(server.options.port).to.equal(port);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with host', function () {
                var host = 'localhost';
                server = server_1.default({ host: host });
                expect(server.options.host).to.equal(host);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with spec version 1', function () {
                server = server_1.default({ spec: 1 });
                expect(server.options.spec).to.equal(1);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with spec version 2', function () {
                server = server_1.default({ spec: 2 });
                expect(server.options.spec).to.equal(2);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with log', function () {
                var logPath = path.resolve(absolutePath, 'log.txt');
                server = server_1.default({ log: logPath });
                expect(server.options.log).to.equal(logPath);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with consumer name', function () {
                var consumerName = 'cName';
                server = server_1.default({ consumer: consumerName });
                expect(server.options.consumer).to.equal(consumerName);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with provider name', function () {
                var providerName = 'pName';
                server = server_1.default({ provider: providerName });
                expect(server.options.provider).to.equal(providerName);
                return expect(server.start()).to.eventually.be.fulfilled;
            });
            it('should start correctly with monkeypatch', function () {
                var s = server_1.default({ monkeypatch: monkeypatchFile });
                expect(s.options.monkeypatch).to.equal(monkeypatchFile);
                return expect(s.start()).to.eventually.be.fulfilled;
            });
            context('Paths', function () {
                it('should start correctly with dir, absolute path', function () {
                    server = server_1.default({ dir: relativePath });
                    expect(server.options.dir).to.equal(absolutePath);
                    return expect(server.start()).to.eventually.be.fulfilled;
                });
                it('should start correctly with log, relative paths', function () {
                    var logPath = path.join(relativePath, 'log.txt');
                    server = server_1.default({ log: logPath });
                    expect(server.options.log).to.equal(path.resolve(logPath));
                    return expect(server.start()).to.eventually.be.fulfilled;
                });
                it('should start correctly with custom ssl cert/key, relative paths', function () {
                    server = server_1.default({
                        sslcert: relativeSSLCertPath,
                        sslkey: relativeSSLKeyPath,
                    });
                    expect(server.options.sslcert).to.equal(absoluteSSLCertPath);
                    expect(server.options.sslkey).to.equal(absoluteSSLKeyPath);
                    return expect(server.start()).to.eventually.be.fulfilled;
                });
            });
            context('File Write Mode', function () {
                it("should start correctly with 'overwrite'", function () {
                    return expect(server_1.default({
                        pactFileWriteMode: 'overwrite',
                    }).start()).to.eventually.be.fulfilled;
                });
                it("should start correctly with 'merge'", function () {
                    return expect(server_1.default({
                        pactFileWriteMode: 'merge',
                    }).start()).to.eventually.be.fulfilled;
                });
                it("should start correctly with 'update'", function () {
                    return expect(server_1.default({
                        pactFileWriteMode: 'update',
                    }).start()).to.eventually.be.fulfilled;
                });
            });
            context('Log Level', function () {
                it("should start correctly with 'debug'", function () {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'debug',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'DEBUG',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
                it("should start correctly with 'info'", function () {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'info',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'INFO',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
                it("should start correctly with 'warn'", function () {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'warn',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'WARN',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
                it("should start correctly with 'error'", function () {
                    return q.allSettled([
                        expect(server_1.default({
                            logLevel: 'error',
                        }).start()).to.eventually.be.fulfilled,
                        expect(server_1.default({
                            logLevel: 'ERROR',
                        }).start()).to.eventually.be.fulfilled,
                    ]);
                });
            });
        });
        it('should dispatch event when starting', function (done) {
            server = server_1.default();
            server.once('start', function () { return done(); });
            server.start();
        });
        it('should change running state to true', function () {
            server = server_1.default();
            return server.start().then(function () { return expect(server['__running']).to.be.true; });
        });
    });
    describe('Stop server', function () {
        context('when already started', function () {
            it('should stop running', function () {
                server = server_1.default();
                return server.start().then(function () { return server.stop(); });
            });
            it('should dispatch event when stopping', function (done) {
                server = server_1.default();
                server.once('stop', function () { return done(); });
                server.start().then(function () { return server.stop(); });
            });
            it('should change running state to false', function () {
                server = server_1.default();
                return server
                    .start()
                    .then(function () { return server.stop(); })
                    .then(function () { return expect(server['__running']).to.be.false; });
            });
        });
    });
    describe('Delete server', function () {
        context('when already running', function () {
            it('should stop & delete server', function () {
                server = server_1.default();
                return server.start().then(function () { return server.delete(); });
            });
            it('should dispatch event when deleting', function (done) {
                server = server_1.default();
                server.once('delete', function () { return done(); });
                server.start().then(function () { return server.delete(); });
            });
            it('should change running state to false', function () {
                server = server_1.default();
                return server
                    .start()
                    .then(function () { return server.delete(); })
                    .then(function () { return expect(server['__running']).to.be.false; });
            });
        });
    });
});
//# sourceMappingURL=server.spec.js.map